; File: aes3mon.asm
; code for AES3 monitor project
; 6/27/07 Initial work, started with ceripod.ASM file
; 6/29 Hardware is running.  Locking to CD S/PDIF, status looks good.  Cool!
; 6/30 Now have some status bits displayed and sample rate.
; 7/1 Have all 4 lines of info on LCD.  Now adding CSSR bits display.
; 7/3 Adding ratio register values for upper sample rates
; 7/15 Have now added switch to select normal/chAstat/chBstat as fourth line
; Use tab setting of 8 if using PFE to look at this file
; (under current Modes, Text formatting)
; 7/21 Changing aux display to number of bits.
; 7/22 Adding code for level meter display.  Put in table lookup for LED's.
;  can't seem to get 9th data bit to work.  Maybe overrun is trashing it?
; 7/23 Fixed two problems.  Seems that RX9 must be read before disbling CREN.
;      Also, I was not setting 8416 to 1 ck delay for I2S mode, so data was wrong.
;      Now it seems to match Flying Cow LED's.
; 7/25 Adding -15 dB level.  Red is now anything at or over 255.  
;      Got second channel working.  Need to add peak detection, and process neg samples.
; 7/27 Changed to look-up table for shift, bit reversing, and absolute value conversion
;      Now neg and pos samples are processed the same.  Faster, seems to work.
; 7/28 Adding code to peak detect in all four time slots, only two values are displayed
; 7/29 Added peak resetting based on timer 1.  Rate matches Flying Cow.  Enabled Red LED's.
;      Made PB sw into interrupt source.  Now I can exit from meter mode.
;      Changed length display to use word length bits in channel status.
; 7/30 Changing code to reflect AP Pro mode wd length bits
;      Added define for whether or not we have meter ckt
; 8/04 A bunch of changes to metering code to make it faster
;      Tried leaving cont rec enabled all the time.  That killed it!
; 8/23 Changing to a different PIC, and new PC bd
;      Now we have two PICs.  This is the code for the chip that talks to the LCD module.
; 8/29 Changing initDIR to remove one clock data delay so we don't need FF
;      Adding readback code for LCD module so we can try to detect when it's hosed.
;      Adding more ratio register values for 32K sample rate.
; 8/30 Changing timing to reflect new clk rate

; To do: enable BOR and test it.

        LIST    P=18F2221	;PIC18F2221 is the target processor
			
;			;include std definitions for our part
#include "P18F2221.INC"

	CONFIG OSC=HS, FCMEN=OFF, IESO=OFF
	CONFIG PWRT=ON, BOR=SOFT, BORV=1
	CONFIG WDT=OFF, WDTPS=32768
	CONFIG CCP2MX=RC1, LPT1OSC=ON, PBADEN=DIG, MCLRE=OFF
	CONFIG STVREN=OFF, LVP=OFF, XINST=OFF
	CONFIG BBSIZ=BB256, DEBUG=OFF
	CONFIG CP0=OFF, CP1=OFF
	CONFIG CPD=OFF, CPB=OFF
	CONFIG WRT0=OFF, WRT1=OFF
	CONFIG WRTC=OFF, WRTB=OFF, WRTD=OFF
	CONFIG EBTR0=OFF, EBTR1=OFF, EBTRB=OFF

; resulting config words:
;		300001	02
;		300002	0A
;		300003	1E
;		300005	05
;		300006	80
;		300008	03
;		300009	C0
;		30000A	03
;		30000B	E0
;		30000C	03
;		30000D	40

;#define sim 1
;#define dmpenab 1

temp1		equ	20
ttemp1		equ	21	;used by delay routines
ttemp2		equ	22	;used by delay routines
tbitcnt		equ	23	;used by delay routines
lcddat		equ	24
lcdaddr		equ	25	;DDRAM address
lcdrow		equ	26	;starting row
lcdcol		equ	27	;starting col on that row
regaddr		equ	28
regdata		equ	29
SPIwdat		equ	2A
SPIrdat		equ	2B
ASCtemp		equ	2C
bitctr		equ	2D
flshrdatl	equ	2E
flshrdath	equ	2F
flashaddl	equ	30
flashaddh	equ	31
flags		equ	32
trycnt		equ	33
unlckcnt	equ	34
savecs4		equ	35
numbits		equ	36	;24 - #aux bits
swlckout	equ	37	;# of loops to ignore sw input
lentemp		equ	38
portdata	equ	39
timeout		equ	3A

BSR_TEMP	equ	7D
W_TEMP		equ	7E	;almost top of access bank
STATUS_TEMP	equ	7F	;top of access bank

#define SPDIF	flags,0		;0=XLR selected, 1=S/PDIF
#define PROFLG	flags,1		;1=prof mode
#define NORMAL	flags,2		;line 4 normal mode
#define ACHST	flags,3		;line 4 is A ch st bytes 0-4
#define BCHST	flags,4		;line 4 is B ch st bytes 0-4
#define SWDN	flags,5		;pb int occured

;define our I/O port signals:

#define LCDD0	PORTA,0
#define LCDD1	PORTA,1
#define	LCDD2	PORTA,2		
#define LCDD3	PORTA,3	
#define SPARE1	PORTA,4
#define SPARE2	PORTA,5

#define PORTADIR 0x00		;all port A pins are outputs

#define SWIN0	PORTB,0		;input
#define SPARE3	PORTB,1		;output
#define LCDRW	PORTB,2		;output
#define LCDE	PORTB,3		;output
#define LCDRS	PORTB,4		;output
#define PGMPIN	PORTB,5		;input
#define PGCPIN	PORTB,6		;input
#define PGDPIN	PORTB,7		;input

#define PORTBDIR 0xE1		;port B pins

#define CSb	PORTC,0		;output
#define MTREN	PORTC,1		;output
#define MTRMDE	PORTC,2		;output
#define SPISCK	PORTC,3		;output
#define	MISO	PORTC,4		;input	
#define MOSI	PORTC,5		;output
#define SPARE4	PORTC,6		;output
#define DIRRESb	PORTC,7		;output

#define PORTCDIR 0x10		;port C pins

	org	0000h
	goto	start


myISR:
	org	00018h			;all ints vector here
	movwf	W_TEMP 			;W_TEMP is in virtual bank
	movff	STATUS,STATUS_TEMP 	;STATUS_TEMP located anywhere
	movff	BSR,BSR_TEMP 		;BSR_TMEP located anywhere

	bcf	INTCON,INT0IF		;clear flag
	bsf	SWDN

	movff	BSR_TEMP,BSR 		;restore BSR
	movf	W_TEMP,W 		;restore WREG
	movff	STATUS_TEMP,STATUS 	;restore STATUS
	retfie



				;this routine is out of order, to insure that
				;it doesn't cross a 256-byte boundary
getASC:	andlw	0x0F		;just use low nibble
	movwf	ASCtemp
	addwf	ASCtemp,W	;double orig value to get index
	addwf	PCL,F		;returns ASCII for nibble in W
	retlw	A'0'	
	retlw	A'1'
	retlw	A'2'
	retlw	A'3'	
	retlw	A'4'
	retlw	A'5'
	retlw	A'6'	
	retlw	A'7'
	retlw	A'8'
	retlw	A'9'	
	retlw	A'A'
	retlw	A'B'
	retlw	A'C'	
	retlw	A'D'
	retlw	A'E'
	retlw	A'F'

				;this routine is out of order, to insure that
				;it doesn't cross a 256-byte boundary
getrsa:				;get starting DD address based on row number
	rlncf	lcdrow,W	;get index into w
	addwf	PCL,F		;this is set up for a 20 char/row LCD
	retlw	0x00		;one
	retlw	0x40		;two
	retlw	0x14		;three
	retlw	0x54		;four



getwlenc:			;look up table for word length decoding in consumer mode
	andlw	0x0F		;just use low nibble
	movwf	lentemp
	addwf	lentemp,W	;double orig value to get index
	addwf	PCL,F		;returns length value for nibble in W
	retlw	D'0'		;0x00
	retlw	D'0'		;0x01		
	retlw	D'16'		;0x02
	retlw	D'20'		;0x03		
	retlw	D'18'		;0x04
	retlw	D'22'		;0x05		
	retlw	D'0'		;0x06
	retlw	D'0'		;0x07		
	retlw	D'19'		;0x08
	retlw	D'23'		;0x09		
	retlw	D'20'		;0x0A
	retlw	D'24'		;0x0B		
	retlw	D'17'		;0x0C
	retlw	D'21'		;0x0D		
	retlw	D'0'		;0x0E
	retlw	D'0'		;0x0F		
	

getwlenp20:			;look up table for word length decoding in pro mode max 20 bits
	andlw	0x07		;just use low 3 bits
	movwf	lentemp
	addwf	lentemp,W	;double orig value to get index
	addwf	PCL,F		;returns length value for value in W
	retlw	D'0'		;0x00 this table uses Sencore data, not AN22
	retlw	D'16'		;0x01 does not agree with AN22			
	retlw	D'18'		;0x02
	retlw	D'0'		;0x03		
	retlw	D'19'		;0x04
	retlw	D'20'		;0x05		
	retlw	D'17'		;0x06
	retlw	D'0'		;0x07		


getwlenp24:			;look up table for word length decoding in pro mode max 24 bits
	andlw	0x07		;just use low 3 bits
	movwf	lentemp
	addwf	lentemp,W	;double orig value to get index
	addwf	PCL,F		;returns length value for value in W
	retlw	D'0'		;0x00 this table uses Sencore data
	retlw	D'20'		;0x01 does not agree with AN22		
	retlw	D'22'		;0x02
	retlw	D'0'		;0x03		
	retlw	D'23'		;0x04
	retlw	D'24'		;0x05		
	retlw	D'21'		;0x06
	retlw	D'0'		;0x07		

	;define some null-terminated strings we will need:
ConsStr:
	dw	"Cons \x00"
CpyRStr:
	dw	"(c)  \x00"
CSSRStr:
	dw	"CSSR:\x00"
DataLenStr:
	dw	" Len:\x00"
EmphStr:
	dw	"Emph \x00"
ErrStr:
	dw 	"Errs: \x00"
FiveSpStr:
	dw	"     \x00"
InitStr:
	dw	"Dig Aud Monitor V2.0\x00"
NoLockStr:
	dw	"No lock, scanning...\x00"
OrigStr:
	dw	"Orig \x00"
ProStr:
	dw	"Pro  \x00"
regStr:
	dw	"Reg: \x00"
Str32:
	dw	"32 KHz  \x00"
Str44:
	dw	"44.1 KHz\x00"
Str48:
	dw	"48 KHz  \x00"
Str88:
	dw	"88.2 KHz\x00"
Str96:
	dw	"96 KHz  \x00"
Str176:
	dw	"176 KHz \x00"
Str192:
	dw	"192 KHz \x00"
XLockStr:
	dw	"XLR Lock SR:\x00"
XNoLockStr:
	dw	"XLR      SR:\x00"
SLockStr:
	dw	"RCA Lock SR:\x00"
SNoLockStr:
	dw	"RCA      SR:\x00"
ClrStr:
	dw	"                    \x00"



docurpos:			;updates module cursor position, based on byte in lcddat
	movlw	0x1F		;col is in low 5 bits
	andwf	lcddat,W
	movwf	lcdcol		;update column
	swapf	lcddat,F	;row is in bits 6,5
	rrncf	lcddat,W
	andlw	0x03
	movwf	lcdrow
	rcall	setaddr		;set module address based on new values
	retlw	0x00

dsperrs:			;fill in third line of LCD
	movlw	0xC0
	rcall	wrLCDdat	;set cursor position to row 2, col 0

	movlw	0x0C		;check error bits
	movwf	regaddr
	rcall	ReadReg

	movlw	LOW ErrStr
	movwf	flashaddl
	movlw	HIGH ErrStr
	movwf	flashaddh	
	rcall	sendFstr

	movlw	' '
	btfsc	SPIrdat,6	;QCRC bit
	movlw	'Q'
	rcall	wrLCDdat

	movlw	' '
	btfsc	SPIrdat,5	;CCRC bit
	movlw	'C'
	rcall	wrLCDdat

	movlw	' '
	btfsc	SPIrdat,4	;Unlock bit
	movlw	'U'
	rcall	wrLCDdat

	movlw	' '
	btfsc	SPIrdat,3	;V bit
	movlw	'V'
	rcall	wrLCDdat

	movlw	' '
	btfsc	SPIrdat,2	;C bit
	movlw	'C'
	rcall	wrLCDdat

	movlw	' '
	btfsc	SPIrdat,1	;Bi-phase bit
	movlw	'B'
	rcall	wrLCDdat

	movlw	' '
	btfsc	SPIrdat,0	;Parity bit
	movlw	'P'
	rcall	wrLCDdat

	movlw	LOW DataLenStr
	movwf	flashaddl
	movlw	HIGH DataLenStr
	movwf	flashaddh	
	rcall	sendFstr

	btfsc	PROFLG
	bra	promde
consmde:
	movlw	0x1D		;ch stat byte #4
	movwf	regaddr
	rcall	ReadReg
	movf	SPIrdat,W
	rcall	getwlenc	;get word length from table
	bra	wldorest
promde:
	movlw	0x01B		;ch stat byte #2
	movwf	regaddr
	rcall	ReadReg
	movlw	0x07		;aux bit use field
	andwf	SPIrdat,W
	xorlw	0x04
	bz	max24
	movlw	0x07
	andwf	SPIrdat,W
	bnz	wlntval
max20:	rrncf	SPIrdat,F	;aux use = 000
	rrncf	SPIrdat,F
	rrncf	SPIrdat,W
	rcall	getwlenp20	;get word length from max 20 pro table
	bra	wldorest
max24:	rrncf	SPIrdat,F	;aux use = 001
	rrncf	SPIrdat,F
	rrncf	SPIrdat,W
	rcall	getwlenp24	;get word length from max 24 pro table
	bra	wldorest	
wlntval:
	movlw	0x00		;no valid word length
wldorest:	
	movwf	numbits		;set up numbits as 24
	movlw	D'20'		;convert result into decimal for display
	subwf	numbits,W
	btfss	STATUS,C
	bra	ntgt20
	movlw	D'20'
	subwf	numbits,F	;subtract out tens part
	movlw	'2'
	rcall	wrLCDdat	
	bra	skiptens	
ntgt20: movlw	D'10'
	subwf	numbits,W
	btfss	STATUS,C
	bra	skiptens
	movlw	D'10'
	subwf	numbits,F	;subtract out tens part
	movlw	'1'
	rcall	wrLCDdat	;convert #into decimal for display
skiptens:
	movf	numbits,W
	rcall	getASC
	rcall	wrLCDdat	;write out # of bits second digit
	return
	

dspflags:			;fill in second line of LCD
	movlw	0xA0
	rcall	wrLCDdat	;set cursor position to row 1, col 0

	movlw	0x0A		;check flags
	movwf	regaddr
	rcall	ReadReg
	
	btfsc	SPIrdat,3	;check cons/pro
	bra	promode
cons:
	bcf	PROFLG
	movlw	LOW ConsStr	;select 'cons' 
	movwf	flashaddl
	movlw	HIGH ConsStr
	movwf	flashaddh	
	bra	skip1df
promode:
	bsf	PROFLG
	movlw	LOW ProStr	;select 'pro' 
	movwf	flashaddl
	movlw	HIGH ProStr
	movwf	flashaddh	
skip1df:
	rcall	sendFstr

	btfsc	SPIrdat,0	;check emph
	bra	noemph

	movlw	LOW EmphStr	;select 'Emph' 
	movwf	flashaddl
	movlw	HIGH EmphStr
	movwf	flashaddh	
	bra	skip2df
noemph:
	movlw	LOW FiveSpStr	;select spaces
	movwf	flashaddl
	movlw	HIGH FiveSpStr
	movwf	flashaddh	
skip2df:
	rcall	sendFstr

	btfss	SPIrdat,1	;check orig bit
	bra	ntorig

	movlw	LOW OrigStr	;select 'Orig' 
	movwf	flashaddl
	movlw	HIGH OrigStr
	movwf	flashaddh	
	bra	skip3df
ntorig:
	movlw	LOW FiveSpStr	;select spaces
	movwf	flashaddl
	movlw	HIGH FiveSpStr
	movwf	flashaddh	
skip3df:
	rcall	sendFstr

	btfsc	SPIrdat,2	;check copy bit
	bra	ntcopy

	movlw	LOW CpyRStr	;select '(c)' 
	movwf	flashaddl
	movlw	HIGH CpyRStr
	movwf	flashaddh	
	bra	skip4df
ntcopy:
	movlw	LOW FiveSpStr	;select spaces
	movwf	flashaddl
	movlw	HIGH FiveSpStr
	movwf	flashaddh	
skip4df:
	rcall	sendFstr
	return
	

dspsmprate:
	movlw	0x018		;read sample rate ratio register
	movwf	regaddr
	rcall	ReadReg

	movlw	0x61
	subwf	SPIrdat,W
	bz	sr32k

	movlw	0x60
	subwf	SPIrdat,W
	bz	sr32k

	movlw	0x5F
	subwf	SPIrdat,W
	bz	sr32k

	movlw	0x46
	subwf	SPIrdat,W
	bz	sr44k

	movlw	0x45
	subwf	SPIrdat,W
	bz	sr44k

	movlw	0x40
	subwf	SPIrdat,W
	bz	sr48k

	movlw	0x3F
	subwf	SPIrdat,W
	bz	sr48k

	movlw	0x23
	subwf	SPIrdat,W
	bz	sr88k
	
	movlw	0x22
	subwf	SPIrdat,W
	bz	sr88k

	movlw	0x20
	subwf	SPIrdat,W
	bz	sr96k
	
	movlw	0x12
	subwf	SPIrdat,W
	bz	sr176k
	
	movlw	0x11
	subwf	SPIrdat,W
	bz	sr176k
	
	movlw	0x10
	subwf	SPIrdat,W
	bz	sr192k
	
	movlw	LOW regStr	;send out 'Reg: ' 
	movwf	flashaddl
	movlw	HIGH regStr
	movwf	flashaddh	
	rcall	sendFstr

	swapf	SPIrdat,W	;send out reg value if it's not one we expect
	rcall	getASC
	rcall	wrLCDdat
	movf	SPIrdat,W
	rcall	getASC
	rcall	wrLCDdat
	movlw	' '		;erase anything that was there
	rcall	wrLCDdat
	return

sr32k:	movlw	LOW Str32
	movwf	flashaddl
	movlw	HIGH Str32
	movwf	flashaddh
	bra	doresds

sr44k:	movlw	LOW Str44
	movwf	flashaddl
	movlw	HIGH Str44
	movwf	flashaddh
	bra	doresds

sr48k:	movlw	LOW Str48
	movwf	flashaddl
	movlw	HIGH Str48
	movwf	flashaddh
	bra	doresds

sr88k:	movlw	LOW Str88
	movwf	flashaddl
	movlw	HIGH Str88
	movwf	flashaddh
	bra	doresds

sr96k:	movlw	LOW Str96
	movwf	flashaddl
	movlw	HIGH Str96
	movwf	flashaddh
	bra	doresds

sr176k:	movlw	LOW Str176
	movwf	flashaddl
	movlw	HIGH Str176
	movwf	flashaddh
	bra	doresds

sr192k:	movlw	LOW Str192
	movwf	flashaddl
	movlw	HIGH Str192
	movwf	flashaddh

doresds:	
	rcall	sendFstr
	return


dspbyte:
	movwf	regaddr
	rcall	ReadReg
	swapf	SPIrdat,W
	rcall	getASC
	rcall	wrLCDdat
	movf	SPIrdat,W
	rcall	getASC
	rcall	wrLCDdat
	movlw	' '
	rcall	wrLCDdat
	return	


dspAchst:
	movlw	0xE0		
	rcall	wrLCDdat	;set cursor to row 3, col 0

	movlw	'C'
	rcall	wrLCDdat
	movlw	'H'
	rcall	wrLCDdat
	movlw	' '
	rcall	wrLCDdat
	movlw	'A'
	rcall	wrLCDdat
	movlw	':'
	rcall	wrLCDdat
	movlw	0x19
	rcall	dspbyte
	movlw	0x1A
	rcall	dspbyte
	movlw	0x1B
	rcall	dspbyte
	movlw	0x1C
	rcall	dspbyte
	movlw	0x1D
	rcall	dspbyte
	return


dspBchst:
	movlw	0xE0		
	rcall	wrLCDdat	;set cursor to row 3, col 0

	movlw	'C'
	rcall	wrLCDdat
	movlw	'H'
	rcall	wrLCDdat
	movlw	' '
	rcall	wrLCDdat
	movlw	'B'
	rcall	wrLCDdat
	movlw	':'
	rcall	wrLCDdat
	movlw	0x1E
	rcall	dspbyte
	movlw	0x1F
	rcall	dspbyte
	movlw	0x20
	rcall	dspbyte
	movlw	0x21
	rcall	dspbyte
	movlw	0x22
	rcall	dspbyte
	return
	
	

dspsrbits:			;display channel status sample rate bits
	movlw	0xE0		
	rcall	wrLCDdat	;set cursor to row 3, col 0

	movlw	LOW CSSRStr	;display "CSSR:"
	movwf	flashaddl
	movlw	HIGH CSSRStr
	movwf	flashaddh	
	rcall	sendFstr

	btfsc	PROFLG
	bra	pmode
cmode:	movlw	0x01C		;read chan A chan stat 3 register
	movwf	regaddr
	rcall	ReadReg
	movlw	' '
	rcall	wrLCDdat	;send out a space
	movlw	'0'
	btfsc	SPIrdat,3
	movlw	'1'
	rcall	wrLCDdat	;send out bits to LCD
	movlw	'0'
	btfsc	SPIrdat,2
	movlw	'1'
	rcall	wrLCDdat	;send out bits to LCD
	movlw	'0'
	btfsc	SPIrdat,1
	movlw	'1'
	rcall	wrLCDdat	;send out bits to LCD
	movlw	'0'
	btfsc	SPIrdat,0
	movlw	'1'
	rcall	wrLCDdat	;send out 4 bits to LCD
	movlw	' '
	rcall	wrLCDdat	;then two spaces
	movlw	' '
	rcall	wrLCDdat
	movlw	0x0F
	andwf	SPIrdat,F	;just low 4 bits
	bnz	ntcs44
cs44k:	movlw	LOW Str44
	movwf	flashaddl
	movlw	HIGH Str44
	movwf	flashaddh
	bra	doresdcs
ntcs44:	movlw	0x02
	subwf	SPIrdat,W
	bnz	ntcs48
cs48k:	movlw	LOW Str48
	movwf	flashaddl
	movlw	HIGH Str48
	movwf	flashaddh
	bra	doresdcs
ntcs48:	movlw	0x03
	subwf	SPIrdat,W
	bnz	ntcs32
cs32k:	movlw	LOW Str32
	movwf	flashaddl
	movlw	HIGH Str32
	movwf	flashaddh
	bra	doresdcs
ntcs32:	movlw	0x08
	subwf	SPIrdat,W
	bnz	ntcs88	
cs88k:	movlw	LOW Str88
	movwf	flashaddl
	movlw	HIGH Str88
	movwf	flashaddh
	bra	doresdcs
ntcs88:	movlw	0x0A
	subwf	SPIrdat,W
	bnz	ntcs96
cs96k:	movlw	LOW Str96
	movwf	flashaddl
	movlw	HIGH Str96
	movwf	flashaddh
	bra	doresdcs
ntcs96: movlw	0x0C
	subwf	SPIrdat,W
	bnz	ntcs176
cs176k:	movlw	LOW Str176
	movwf	flashaddl
	movlw	HIGH Str176
	movwf	flashaddh
	bra	doresdcs
ntcs176: 
	movlw	0x0E
	subwf	SPIrdat,W
	bnz	ntcs192
cs192k:	movlw	LOW Str192
	movwf	flashaddl
	movlw	HIGH Str192
	movwf	flashaddh
	bra	doresdcs
ntcs192:	
	return	
doresdcs:
	rcall	sendFstr
	return		
	
pmode:	movlw	0x1D		;read chan A chan stat 4 register
	movwf	regaddr
	rcall	ReadReg
	movf	SPIrdat,W
	movwf	savecs4		;save value
	movlw	0x19		;read chan A chan stat reg 0
	movwf	regaddr
	rcall	ReadReg		;leave chan 0 in SPIrdat
	movlw	'0'
	btfsc	SPIrdat,7
	movlw	'1'
	rcall	wrLCDdat	;send out bits to LCD
	movlw	'0'
	btfsc	SPIrdat,6
	movlw	'1'
	rcall	wrLCDdat
	movlw	'0'
	btfsc	savecs4,6
	movlw	'1'
	rcall	wrLCDdat
	movlw	'0'
	btfsc	savecs4,5
	movlw	'1'
	rcall	wrLCDdat
	movlw	'0'
	btfsc	savecs4,4
	movlw	'1'
	rcall	wrLCDdat
	movlw	'0'
	btfsc	savecs4,3
	movlw	'1'
	rcall	wrLCDdat
	movlw	' '		;then a space
	rcall	wrLCDdat
	movlw	0x78
	andwf	savecs4,F	;check for 0's in bits 6-3
	bnz	r4ntzer
	movlw	0xC0		;now look at CS reg 0
	andwf	SPIrdat,F	;just leave top two bits
	movlw	0x40
	subwf	SPIrdat,W
	bz	cs44k
	movlw	0x80
	subwf	SPIrdat,W
	bz	cs48k
	movlw	0xC0
	subwf	SPIrdat,W
	bz	cs32k
r4ntzer:	
	movlw	0xC0		;now look at CS reg 0
	andwf	SPIrdat,F	;just leave top two bits
	bnz	alldun
	movlw	0x50
	subwf	savecs4,W
	bz	cs88k
	movlw	0x10
	subwf	savecs4,W
	bz	cs96k
	movlw	0x58
	subwf	savecs4,W
	bz	cs176k
	movlw	0x18
	subwf	savecs4,W
	bz	cs192k
alldun:	return

	


fcnset:				;we are still in 8-bit mode when we do this
	bcf	LCDRS		;0 = command mode
	movlw	0x02		;top nibble of fcn set on LCD D7-D4
	movwf	PORTA		
	bsf	LCDE		;raise enable
	bcf	LCDE		;drop enable
	rcall	wt10ms		;make sure it's ready
				;OK, now we should be in 4-bit mode
	movlw	0x02		;top nibble of fcn set on LCD D7-D4
	movwf	PORTA
	bsf	LCDE
	bcf	LCDE
	rcall	wt10ms
	movlw	0x08		;lower nibble of fcn set on LCD D7-D4
	movwf	PORTA		;DL: 4-bit interface
	bsf	LCDE		;N: 2-line display (choices are 1 or 2)
	bcf	LCDE		;F: 5x8 dot chars (not 5x10)
	rcall	wt10ms
	retlw	0x00


				;reads one 16-bit word of Flash into flshrdatl,h registers
flashrd:			;and bumps TBLPTR as well
	TBLRD*+			;read low Flash byte into latch, incr ptr
	movf	TABLAT,W
	movwf	flshrdatl	;save it
			
	TBLRD*+			;read high Flash byte into latch, incr ptr
	movf	TABLAT,W
	movwf	flshrdath	;save it

	return



				;set up an up arrow in 44780 CGRAM char 01
initarrs:	
	movlw	0x48		;set CGRAM address to char 01
	rcall	wrLCDcmd
	movlw	0x04		;row 0
	rcall	wrLCDdat
	movlw	0x0E		;row 1
	rcall	wrLCDdat
	movlw	0x15		;row 2
	rcall	wrLCDdat
	movlw	0x04		;row 3
	rcall	wrLCDdat
	movlw	0x04		;row 4
	rcall	wrLCDdat
	movlw	0x04		;row 5
	rcall	wrLCDdat
	movlw	0x04		;row 6
	rcall	wrLCDdat
	movlw	0x00		;row 7
	rcall	wrLCDdat
				;set up a dn arrow in 44780 CGRAM char 02
	movlw	0x50		;set CGRAM address to char 02
	rcall	wrLCDcmd
	movlw	0x04		;row 0
	rcall	wrLCDdat
	movlw	0x04		;row 1
	rcall	wrLCDdat
	movlw	0x04		;row 2
	rcall	wrLCDdat
	movlw	0x04		;row 3
	rcall	wrLCDdat
	movlw	0x15		;row 4
	rcall	wrLCDdat
	movlw	0x0E		;row 5
	rcall	wrLCDdat
	movlw	0x04		;row 6
	rcall	wrLCDdat
	movlw	0x00		;row 7
	rcall	wrLCDdat
	return



initDIR:			;set up DIR chip
	movlw	0x00
	movwf	regaddr
	movwf	regdata
	rcall	WriteReg	;init reg 0

	movlw	0x01
	movwf	regaddr
	movlw	0x00
	movwf	regdata
	rcall	WriteReg	;init reg 1

	movlw	0x02
	movwf	regaddr
	movlw	0x00
	movwf	regdata
	rcall	WriteReg	;init reg 2

	movlw	0x03
	movwf	regaddr
	movlw	0x00
	movwf	regdata
	rcall	WriteReg	;init reg 3

	movlw	0x04
	movwf	regaddr
	movlw	0x89		;Run, RMCK enabled, input 1 selected
	movwf	regdata
	rcall	WriteReg	;init reg 4

	movlw	0x05
	movwf	regaddr
	movlw	0x80		;Master, 64X, 24-bit, Left just, left ch = LRCK high, 0 ck delay (not I2S)
;	movlw	0x84		;Master, 64X, 24-bit, Left just, left ch = LRCK high, 1 ck delay (I2S)
	movwf	regdata
	rcall	WriteReg	;init reg 5

	movlw	0x06
	movwf	regaddr
	movlw	0x7F		;all errors are unmasked
	movwf	regdata
	rcall	WriteReg	;init reg 6

	movlw	0x07
	movwf	regaddr
	movlw	0x00		;all ints are masked
	movwf	regdata
	rcall	WriteReg	;init reg 7

	movlw	0x08
	movwf	regaddr
	movlw	0x00		;more int stuff
	movwf	regdata
	rcall	WriteReg	;init reg 8

	movlw	0x09
	movwf	regaddr
	movlw	0x00		;more int stuff
	movwf	regdata
	rcall	WriteReg	;init reg 9

	return



initLCD:
	rcall	wt400ms		;wait for LCD controller to be ready
	bcf	LCDRW		;we're never reading from the module	

				;following sequence suggested by Hitachi
	bcf	LCDRS		;0 = command mode
	movlw	0x03		;top nibble of fcn set on LCD D7-D4
	movwf	PORTA		
	bsf	LCDE		;raise enable
	bcf	LCDE		;drop enable
	rcall	wt20ms
	bsf	LCDE		;raise enable
	bcf	LCDE		;drop enable
	rcall	wt10ms
	bsf	LCDE		;raise enable
	bcf	LCDE		;drop enable
	rcall	wt10ms

	rcall	fcnset
	movlw	0x0C		;enable display, disable cursor
	rcall	wrLCDcmd
	rcall	wt10ms
	movlw	0x01		;clear screen
	rcall	wrLCDcmd
	rcall	wt10ms
	movlw	0x06		;entry mode: increment addr, cursor pos, no shift
	rcall	wrLCDcmd
	rcall	wt10ms
	retlw	0x00
	


initser:			;sets up serial port
	movlw	0x10
	movwf	TXSTA		;slave, 8-bit, tx disabled, sync mode
	movlw	0x10
	movwf	BAUDCON		;SCKP=1, idle clk state is high
	clrf	RCSTA	
	bsf	RCSTA,SPEN	;enable ser port, 8-bit, sync, low speed
	bsf	RCSTA,RX9	;enable 9-bit reception
	movf	RCREG,W		;read data fifo
	movf	RCREG,W		;twice
	return



ldtblptr:			;load table pointer used to access Flash array	
	movf 	flashaddl,W	;this is a byte address, probably needs to be word-aligned
	movwf 	TBLPTRL		;LS byte of program address to read
	movf 	flashaddh,W 
	movwf 	TBLPTRH		;MS byte of program address to read
	clrf	TBLPTRU		;bits above A15 all 0
	return



readbusy:			;read LCD module busy flag
	movlw	0x0F		;low 4 bits are now inputs
	movwf	TRISA
	bcf	LCDRS		;low for busy read
	bsf	LCDRW		;we're reading
	rcall	wt3us
	bsf	LCDE
	rcall	wt3us		;wait a bit
	swapf	PORTA,W		;read data
	andlw	0xF0
	movwf	portdata	;save it in high nibble
	bcf	LCDE
	rcall	wt3us		;wait a bit
	bsf	LCDE
	rcall	wt3us
	movf	PORTA,W
	andlw	0x0F
	iorwf	portdata
	bcf	LCDE
	rcall	wt3us
	bcf	LCDRW
	movlw	PORTADIR	
	movwf	TRISA		;set port A dir back to default
	return



ReadReg:
	bcf	CSb		;drop chip select	
	rcall	wt5us
	movlw	0x20		;always 0x10, but in top 7 bits, low bit is 0 for write
	movwf	SPIwdat		;chip address is first thing
	rcall	clkdout
	movf	regaddr,W
	movwf	SPIwdat
	rcall	clkdout
	bsf	CSb		;raise chip select
				;that should set MAP address.  Now we can do read
	rcall	wt5us
	bcf	CSb
	rcall	wt5us
	movlw	0x21		;always 0x10, but in top 7 bits, low bit is 1 for read
	movwf	SPIwdat		;chip address is first thing
	rcall	clkdout
	rcall	clkdin
	bsf	CSb		;raise chip select
	return


clkdin:
	movlw	D'8'
	movwf	bitctr
lp1:	bcf	SPISCK		;drop SPI clk
	rcall	wt5us		;wait a bit
	bcf	STATUS,C
	btfsc	MISO		;test input
	bsf	STATUS,C
	rlcf	SPIrdat,F	;shift in rec'd data
	bsf	SPISCK		;raise SPI clk
	rcall	wt5us		;wait a bit
	decfsz	bitctr,F
	bra	lp1
	return


clkdout:
	movlw	D'8'
	movwf	bitctr
lp2:	bcf	SPISCK		;drop SPI clk
	rcall	wt5us		;wait a bit

	rlcf	SPIwdat,F
	btfss	STATUS,C	;set up MOSI based on next bit
	bra	bitis0	
	bsf	MOSI
	bra	doclk
bitis0: bcf	MOSI
doclk:	call	wt5us		;wait a bit
	bsf	SPISCK		;raise SPI clk
	rcall	wt5us		;wait a bit
	decfsz	bitctr,F
	bra	lp2
	return



sendFstr:			;transmit Flash string pointed to by flashadd to LCD
	rcall	wt2ms		;let anything else finish
	rcall	ldtblptr
nxtwdsF:
	rcall	flashrd		;read next two bytes, bump ptr
	movf	flshrdatl,W	;get low byte
	btfsc	STATUS,Z		
	return
	rcall	wrLCDdat
	movf	flshrdath,W	;get high byte
	btfsc	STATUS,Z		
	return
	rcall	wrLCDdat
	bra	nxtwdsF		;go until we hit a null char



setaddr:			;sets address in LCD module based on row and col
	call	getrsa		;get row starting address (table lookup)
	addwf	lcdcol,W	;add in column
	movwf	lcdaddr
	bsf	lcdaddr,7	;needed to set DDRAM address
	movf	lcdaddr,W
	rcall	wrLCDcmd	;set address to computed value
	rcall	wt10ms
	retlw	0x00



WriteReg:			;SPI bit bangs data byte so it's setup around L->H clk edge
				;regaddr and regdata set up before call
	bcf	CSb		;drop chip select	
	rcall	wt5us
	movlw	0x20		;always 0x10, but in top 7 bits, low bit is 0 for write
	movwf	SPIwdat		;chip address is first thing
	rcall	clkdout
	movf	regaddr,W
	movwf	SPIwdat
	rcall	clkdout
	movf	regdata,W
	movwf	SPIwdat
	rcall	clkdout
	bsf	CSb		;raise chip select	
	return



wrLCDdat:
	movwf	lcddat		;save character
	btfsc	lcddat,7	;check for curpos command
	goto	docurpos	;if bit 7 set, then set addr in LCD module
	swapf	lcddat,W	;high nibble of data in low part of W
	movwf	PORTA
	bsf	LCDRS		;1 = data mode
	bsf	LCDE
	bcf	LCDE
	movf	lcddat,W	;low part of data goes next
	movwf	PORTA
	bsf	LCDE
	bcf	LCDE
;	rcall	wt400uS		;delay 400 uSec
	rcall	wtbusy
	retlw	0x00


wrLCDcmd:
	movwf	lcddat		;save character
	swapf	lcddat,W	;high nibble of data in low part of W
	movwf	PORTA
	bcf	LCDRS		;0 = command mode
	bsf	LCDE
	bcf	LCDE
	movf	lcddat,W	;low part of data goes next
	movwf	PORTA
	bsf	LCDE
	bcf	LCDE
;	rcall	wt400uS		;delay 400 uSec
	rcall	wtbusy
	retlw	0x00



wtbusy:	rcall	wt3us
	movlw	D'250'		;wait for LCD module to go not busy
	movwf	timeout		;time out after 25 mSec
bzychk:	rcall	readbusy
	btfss	portdata,7
	retlw	0x00		;no error
	rcall	wt100us
	decfsz	timeout,F
	bra	bzychk
bzytout:
	rcall	initLCD		;try to reset LCD module if it seems hosed
	rcall	initLCD
	retlw	0xFF		;timed out	



				;general-purpose delay routines
				;these use ttemp1, ttemp2, tbitcnt

wt100ms	movlw	D'250'		;kill 100 mSec
	goto	skip22

wt50ms	movlw	D'125'		;kill 50 mSec
	goto	skip22

wt20ms	movlw	D'50'		;kill 20 mSec
	goto	skip22

wt10ms	movlw	D'25'		;kill 10 mSec
	goto	skip22

wt2ms	movlw	D'5'		;kill 2 mSec
	goto	skip22

wt400uS	movlw	D'1'		;kill 400 uSec
skip22	movwf	ttemp2

wtolp	movlw	D'2'		;middle loop is 0.400 mSec
	movwf	tbitcnt
	
wtilp	movlw	D'122'		;1 cycle, delay for 200.2 uSec (12.288 MHz clock)
	movwf	ttemp1		;1 cycle
dlyLp	nop			;1 cycle
	nop			;1 cycle
	decfsz	ttemp1,F	;1 cycle, 2 if jump is taken
	goto	dlyLp		;2 cycles
				;inner loop total: ((122 * 5) + 5) * 0.3255 = 200.4 uSec

	decfsz	tbitcnt,F	;1	
	goto	wtilp		;2 middle loop until zero

	decfsz	ttemp2,F	;outer loop until zero
	goto	wtolp
	retlw	0x00

wt500ms	rcall	wt100ms
wt400ms	rcall	wt100ms
	rcall	wt100ms
wt200ms	rcall	wt100ms
	rcall	wt100ms
	return


wt100us:
	movlw	D'75'		;1
	movwf	temp1		;1
	
dlylpa:	nop			;1
	decfsz	temp1,F		;1
	bra	dlylpa		;2
				;307 * 0.325 = 99.9 uSec
	nop			;1
	return			;2 + 2 for call
 


wt5us:	movlw	D'2'		;1
	movwf	temp1		;1

dlylpb:	nop			;1
	decfsz	temp1,F		;1
	bra	dlylpb		;2
				;15 * 0.325 = 4.88 uSec
	nop			;1
	return			;2 + 2 for call



wt3us:	nop			;1
	nop			;1
	nop			;1
	nop			;1
	nop			;1
				;9 * 0.325 = 2.93 uSec
	return			;2 + 2 for call



;; Above this line are our subroutines
;; *****************************************************************************
			
				;This is the actual start of the program.
start:	
	clrwdt			

	clrf	INTCON2		;enable PORTB pullups, H->L edge for INT0

	movlw	0xC5
	movwf	T0CON		;timer 0 is 8-bit, 1:64 prescaler

	clrf	INTCON		;init intcon register, shut off all interrupts
	clrf	PIE1		;init PIE1 register, shut off all interrupts
	clrf	PIE2		;init PIE2 register, shut off all interrupts

	movlw	0x00
	movwf	PORTA
	movlw	0x00
	movwf	PORTB
	movlw	0x01		;DIRRESb = 0, MTREN = 0, CSb = 1
	movwf	PORTC		;init port C output data

	movlw	0x0F
	movwf	ADCON1		;set port A pins to digital mode
	movlw	PORTADIR	
	movwf	TRISA		;set up port A dir
	movlw	PORTBDIR	
	movwf	TRISB		;set up port B dir
	movlw	PORTCDIR
	movwf	TRISC		;set up port C dir

	rcall	initLCD		;wait 400 mSec, then init LCD

	bsf	DIRRESb		;un-reset DIR chip
;	rcall	initarrs	;set up arrows in CGRAM
	rcall	wt10ms

	rcall	initDIR		;set up CS8416
;	rcall	initser		;set up sync receive, not enabled yet

        bcf	MTREN		;meter PIC not enabled at start
	bcf	MTRMDE		;not used yet, but clear it anyway
mainlp:
	movlw	0x80
	rcall	wrLCDdat	;set cursor position to row 0, col 0

	movlw	LOW InitStr
	movwf	flashaddl
	movlw	HIGH InitStr
	movwf	flashaddh
	rcall	sendFstr	;send sign-on string to LCD

	rcall	wt500ms
	rcall	wt500ms
	rcall	wt500ms
	rcall	wt500ms
	rcall	wt500ms
	rcall	wt500ms

	bsf	NORMAL		;normal line 4 display
	bcf	ACHST		
	bcf	BCHST

	bcf	INTCON,INT0IF
	bcf	SWDN
	bsf	INTCON,INT0IE	;enable pb sw interrupt
	bcf	RCON,IPEN	;no int priority
	bsf	INTCON,GIE

strtovr:
	movlw	0x01		;clear screen
	rcall	wrLCDcmd
	rcall	wt10ms

	movlw	0x80
	rcall	wrLCDdat	;set cursor position to row 0, col 0

	movlw	LOW NoLockStr
	movwf	flashaddl
	movlw	HIGH NoLockStr
	movwf	flashaddh
	rcall	sendFstr	;send no lock string to LCD

getlock:			;at this time we don't have lock
	bcf	SPDIF		;try XLR input first

	movlw	0x04
	movwf	regaddr
	movlw	0x80		;Run, RMCK enabled, input 0 selected
	movwf	regdata
	rcall	WriteReg	;init reg 4

	movlw	D'10'
	movwf	trycnt

Xlkwt:	movlw	0x0C		;read receiver error register
	movwf	regaddr
	rcall	ReadReg

	btfss	SPIrdat,4	;do we have lock?
	bra	gotlock
	
	rcall	wt10ms
	decfsz	trycnt,F
	bra	Xlkwt

	bsf	SPDIF		;try S/PDIF input

	movlw	0x04
	movwf	regaddr
	movlw	0x89		;Run, RMCK enabled, input 1 selected
	movwf	regdata
	rcall	WriteReg	;init reg 4

	movlw	D'10'
	movwf	trycnt

Slkwt:	movlw	0x0C		;read receiver error register
	movwf	regaddr
	rcall	ReadReg

	btfss	SPIrdat,4	;do we have lock?
	bra	gotlock	
	
	rcall	wt10ms
	decfsz	trycnt,F
	bra	Slkwt		
	bra	getlock		;loop til we get lock

gotlock:
	movf	swlckout,W
	bz	ntlast
	decf	swlckout,F
	bnz	ntlast	
	bcf	INTCON,INT0IF	;lockout is now over
	bsf	INTCON,INT0IE	;so re-enable int, clear flag
ntlast:
	movlw	0x80
	rcall	wrLCDdat	;set cursor position to row 0, col 0

	movlw	0x0C		;check lock status
	movwf	regaddr
	rcall	ReadReg

	btfsc	SPDIF		;which input has signal?
	bra	input1

input0:	btfsc	SPIrdat,4	;check unlock bit
	bra	unlck0
	
	movlw	D'15'
	movwf	unlckcnt	;re-init if we are locked

	movlw	LOW XLockStr
	movwf	flashaddl
	movlw	HIGH XLockStr
	movwf	flashaddh
	rcall	sendFstr	;send X lock string to LCD
	bsf	MTREN		;enable level meter
	bra	skipin1

unlck0:
	movlw	LOW XNoLockStr
	movwf	flashaddl
	movlw	HIGH XNoLockStr
	movwf	flashaddh
	rcall	sendFstr	;send X not lock string to LCD
	decfsz	unlckcnt,F
	bra	skipin1
	bcf	MTREN		;disable level meter
	bra	strtovr		;lost lock

input1:	btfsc	SPIrdat,4	;check unlock bit
	bra	unlck1
	
	movlw	D'15'
	movwf	unlckcnt	;re-init if we are locked

	movlw	LOW SLockStr
	movwf	flashaddl
	movlw	HIGH SLockStr
	movwf	flashaddh
	rcall	sendFstr	;send S lock string to LCD
	bsf	MTREN		;enable level meter
	bra	skipin1

unlck1:
	movlw	LOW SNoLockStr
	movwf	flashaddl
	movlw	HIGH SNoLockStr
	movwf	flashaddh
	rcall	sendFstr	;send S not lock string to LCD

	decfsz	unlckcnt,F
	bra	skipin1
	bcf	MTREN		;disable level meter
	bra	strtovr		;lost lock

skipin1:
	rcall	dspsmprate	;display calculated sample rate
	rcall	dspflags
	rcall	dsperrs
	btfsc	NORMAL
	rcall	dspsrbits	;display chan stat sample rate bits
	btfsc	ACHST
	rcall	dspAchst	;or A or B first five CH Stat bytes
	btfsc	BCHST
	rcall	dspBchst
	rcall	wt50ms
	btfss	SWDN		;is switch down?
	bra	gotlock		;no, just loop
	tstfsz	swlckout
	bra	gotlock		;switch is still locked out

	bcf	SWDN		;clear flag
	movlw	D'6'	
	movwf	swlckout	;disable switch for 300 mSec/6 loop passes
	bcf	INTCON,INT0IE
	bcf	INTCON,INT0IF	;clear flag when we disable	
	btfsc	NORMAL		;step through our modes
	bra	itsnrm
	btfsc	ACHST
	bra	itscha

itschb:	bcf	BCHST
	bsf	NORMAL
	bra	gotlock

itscha:	bcf	ACHST
	bsf	BCHST
	bra	gotlock

itsnrm:	bcf	NORMAL
	bsf	ACHST
	bra	gotlock

	END


		



	
	


