; File: aes3mtr.asm
; code for meter part of AES3 monitor project
; 6/27/07 Initial work, started with ceripod.ASM file
; 6/29 Hardware is running.  Locking to CD S/PDIF, status looks good.  Cool!
; 6/30 Now have some status bits displayed and sample rate.
; 7/1 Have all 4 lines of info on LCD.  Now adding CSSR bits display.
; 7/3 Adding ratio register values for upper sample rates
; 7/15 Have now added switch to select normal/chAstat/chBstat as fourth line
; Use tab setting of 8 if using PFE to look at this file
; (under current Modes, Text formatting)
; 7/21 Changing aux display to number of bits.
; 7/22 Adding code for level meter display.  Put in table lookup for LED's.
;  can't seem to get 9th data bit to work.  Maybe overrun is trashing it?
; 7/23 Fixed two problems.  Seems that RX9 must be read before disbling CREN.
;      Also, I was not setting 8416 to 1 ck delay for I2S mode, so data was wrong.
;      Now it seems to match Flying Cow LED's.
; 7/25 Adding -15 dB level.  Red is now anything at or over 255.  
;      Got second channel working.  Need to add peak detection, and process neg samples.
; 7/27 Changed to look-up table for shift, bit reversing, and absolute value conversion
;      Now neg and pos samples are processed the same.  Faster, seems to work.
; 7/28 Adding code to peak detect in all four time slots, only two values are displayed
; 7/29 Added peak resetting based on timer 1.  Rate matches Flying Cow.  Enabled Red LED's.
;      Made PB sw into interrupt source.  Now I can exit from meter mode.
;      Changed length display to use word length bits in channel status.
; 7/30 Changing code to reflect AP Pro mode wd length bits
;      Added define for whether or not we have meter ckt
; 8/04 A bunch of changes to metering code to make it faster
;      Tried leaving cont rec enabled all the time.  That killed it!
; 8/23 Changing to a different PIC, and new PC bd, now have two PICs and two source files
;      This is the code for the level meter PIC 
; 8/30 Changed port init values to eliminate power-on LED flashing
; 8/31 Fixed several bugs.  Now it's working in the hardware.  Only tested up to 48 so far.
; 9/10 This version works at 176, and sort of works at 192, but not completely.
;      Lower bits in right data bleed into left, and visa versa.
; 9/11 Had to read RCREG twice when running at 192 KHz, as sometimes we got two
;      9-bit words before disabling clock.  This version seems OK at 192 and below.

; To do: Enable BOR and test it.

        LIST    P=18F2221	;PIC18F2221 is the target processor

; Meter timing: 32 KHz:   31.25 uSec/2 = 15.63 uSec or 156 instructions, BClk = 2.048 MHz   
;		44.1 KHz: 22.68 uSec/2 = 11.34 uSec or 113 instructions, BClk = 2.822 MHz
;               48 KHz:   20.83 uSec/2 = 10.42 uSec or 104 instructions, BClk = 3.072 MHz
;               88.2 KHz: 11.35 uSec/2 = 5.67 uSec or 56 instructions, BClk = 5.645 MHz
; 		96 KHz: 10.42 uSec/2 = 5.21 uSec or 52 instructions, BClk = 6.144 MHz
;               176.4 KHz: 5.67 uSec/2 = 2.83 uSec or 28 instructions, BClk = 11.29 MHz
;               192.0 KHz: 5.208 uSec/2 = 2.604 uSec or 26 instructions, BClk = 12.29 MHz	
			
; include std definitions for our part
#include "P18F2221.INC"

	CONFIG OSC=HSPLL, FCMEN=OFF, IESO=OFF
	CONFIG PWRT=ON, BOR=SOFT, BORV=1
	CONFIG WDT=OFF, WDTPS=32768
	CONFIG CCP2MX=RC1, LPT1OSC=ON, PBADEN=DIG, MCLRE=OFF
	CONFIG STVREN=OFF, LVP=OFF, XINST=OFF
	CONFIG BBSIZ=BB256, DEBUG=OFF
	CONFIG CP0=ON, CP1=ON
	CONFIG CPD=OFF, CPB=ON
	CONFIG WRT0=OFF, WRT1=OFF
	CONFIG WRTC=OFF, WRTB=OFF, WRTD=OFF
	CONFIG EBTR0=OFF, EBTR1=OFF, EBTRB=OFF

; resulting config words:
;		300001	06
;		300002	0A
;		300003	1E
;		300005	05
;		300006	80
;		300008	00
;		300009	80
;		30000A	03
;		30000B	E0
;		30000C	03
;		30000D	40

;#define sim 1
;#define dmpenab 1

temp1		equ	20
ttemp1		equ	21	;used by delay routines
ttemp2		equ	22	;used by delay routines
tbitcnt		equ	23	;used by delay routines
flshrdatl	equ	24
flshrdath	equ	25
flashaddl	equ	26
flashaddh	equ	27
flags		equ	28
rgtpeak		equ	29
lftpeak		equ	2A
bit9temp	equ	2B

BSR_TEMP	equ	7D
W_TEMP		equ	7E	;almost top of access bank
STATUS_TEMP	equ	7F	;top of access bank

RAMtab		equ	0x100	;copy of revtab0 sits in RAM, from 100-200

#define GOTINT0	flags,7		;MTREN int occured

;define our I/O port signals:

#define FLIPGATE	PORTA,0	;output
#define LEDRGTO		PORTA,1	;output
#define LEDLFTO		PORTA,2	;output
#define SPARE1		PORTA,3	;output
#define LEDENRb		PORTA,4	;output	
#define LEDENLb 	PORTA,5	;output

#define PORTADIR 0x00		;all port A pins are outputs

#define MTREN		PORTB,0	;input
#define MTRMDE		PORTB,1	;input
#define LRCK		PORTB,2	;input
#define SPARE2		PORTB,3	;output
#define SPARE3		PORTB,4	;output
#define PGMPIN		PORTB,5	;input
#define PGCPIN		PORTB,6	;output, also LED6
#define PGDPIN		PORTB,7	;output, also LED7

#define PORTBDIR 0x27		;port B pins

#define LED0	PORTC,0		;output
#define LED1	PORTC,1		;output
#define LED2	PORTC,2		;output
#define LED3	PORTC,3		;output
#define	LED4	PORTC,4		;output	
#define LED5	PORTC,5		;output
#define SERCLK	PORTC,6		;input
#define SERDATA	PORTC,7		;input

#define PORTCDIR 0xC0		;port B pins

	org	0000h
	goto	start


myISR:
	org	00018h			;all ints vector here
	movwf	W_TEMP 			;W_TEMP is in virtual bank
	movff	STATUS,STATUS_TEMP 	;STATUS_TEMP located anywhere
	movff	BSR,BSR_TEMP 		;BSR_TMEP located anywhere

	bcf	INTCON,INT0IF		;clear flag
	bsf	LEDENRb			;disable LED drives right away
	bsf	LEDENLb
	reset				;start over, with meter disabled, whenever MTREN drops

	movff	BSR_TEMP,BSR 		;restore BSR
	movf	W_TEMP,W 		;restore WREG
	movff	STATUS_TEMP,STATUS 	;restore STATUS
	retfie


	org	0x100
revtab:			;lookup table performs shift, reverses bits in a byte, if LSB=0
			;also does absolute value conversion
			;copied into RAM for faster access
;		  0    1    2    3    4    5    6    7    8    9    A    B    C    D    E    F
	db	0x00,0xFF,0x80,0x7F,0x40,0xBF,0xC0,0x3F,0x20,0xDF,0xA0,0x5F,0x60,0x9F,0xE0,0x1F		;0x
	db	0x10,0xEF,0x90,0x6F,0x50,0xAF,0xD0,0x2F,0x30,0xCF,0xB0,0x4F,0x70,0x8F,0xF0,0x0F		;1x
	db	0x08,0xF7,0x88,0x77,0x48,0xB7,0xC8,0x37,0x28,0xD7,0xA8,0x57,0x68,0x97,0xE8,0x17		;2x
	db	0x18,0xE7,0x98,0x67,0x58,0xA7,0xD8,0x27,0x38,0xC7,0xB8,0x47,0x78,0x87,0xF8,0x07		;3x	
	db	0x04,0xFB,0x84,0x7B,0x44,0xBB,0xC4,0x3B,0x24,0xDB,0xA4,0x5B,0x64,0x9B,0xE4,0x1B		;4x
	db	0x14,0xEB,0x94,0x6B,0x54,0xAB,0xD4,0x2B,0x34,0xCB,0xB4,0x4B,0x74,0x8B,0xF4,0x0B		;5x	
	db	0x0C,0xF3,0x8C,0x73,0x4C,0xB3,0xCC,0x33,0x2C,0xD3,0xAC,0x53,0x6C,0x93,0xEC,0x13		;6x	
	db	0x1C,0xE3,0x9C,0x63,0x5C,0xA3,0xDC,0x23,0x3C,0xC3,0xBC,0x43,0x7C,0x83,0xFC,0x03		;7x
	db	0x02,0xFD,0x82,0x7D,0x42,0xBD,0xC2,0x3D,0x22,0xDD,0xA2,0x5D,0x62,0x9D,0xE2,0x1D		;8x
	db	0x12,0xED,0x92,0x6D,0x52,0xAD,0xD2,0x2D,0x32,0xCD,0xB2,0x4D,0x72,0x8D,0xF2,0x0D		;9x	
	db	0x0A,0xF5,0x8A,0x75,0x4A,0xB5,0xCA,0x35,0x2A,0xD5,0xAA,0x55,0x6A,0x95,0xEA,0x15		;Ax
	db	0x1A,0xE5,0x9A,0x65,0x5A,0xA5,0xDA,0x25,0x3A,0xC5,0xBA,0x45,0x7A,0x85,0xFA,0x05		;Bx
	db	0x06,0xF9,0x86,0x79,0x46,0xB9,0xC6,0x39,0x26,0xD9,0xA6,0x59,0x66,0x99,0xE6,0x19		;Cx
	db	0x16,0xE9,0x96,0x69,0x56,0xA9,0xD6,0x29,0x36,0xC9,0xB6,0x49,0x76,0x89,0xF6,0x09		;Dx
	db	0x0E,0xF1,0x8E,0x71,0x4E,0xB1,0xCE,0x31,0x2E,0xD1,0xAE,0x51,0x6E,0x91,0xEE,0x11		;Ex
	db	0x1E,0xE1,0x9E,0x61,0x5E,0xA1,0xDE,0x21,0x3E,0xC1,0xBE,0x41,0x7E,0x81,0xFE,0x01		;Fx

revtab1:			;lookup table performs shift, reverses bits in a byte, if LSB=1
				;also does absolute value conversion
				;this table is no longer used
;		  0    1    2    3    4    5    6    7    8    9    A    B    C    D    E    F
	db	0x01,0xFE,0x81,0x7E,0x41,0xBE,0xC1,0x3E,0x21,0xDE,0xA1,0x5E,0x61,0x9E,0xE1,0x1E		;0x
	db	0x11,0xEE,0x91,0x6E,0x51,0xAE,0xD1,0x2E,0x31,0xCE,0xB1,0x4E,0x71,0x8E,0xF1,0x0E		;1x
	db	0x09,0xF6,0x89,0x76,0x49,0xB6,0xC9,0x36,0x29,0xD6,0xA9,0x56,0x69,0x96,0xE9,0x16		;2x
	db	0x19,0xE6,0x99,0x66,0x59,0xA6,0xD9,0x26,0x39,0xC6,0xB9,0x46,0x79,0x86,0xF9,0x06		;3x	
	db	0x05,0xFA,0x85,0x7A,0x45,0xBA,0xC5,0x3A,0x25,0xDA,0xA5,0x5A,0x65,0x9A,0xE5,0x1A		;4x
	db	0x15,0xEA,0x95,0x6A,0x55,0xAA,0xD5,0x2A,0x35,0xCA,0xB5,0x4A,0x75,0x8A,0xF5,0x0A		;5x	
	db	0x0D,0xF2,0x8D,0x72,0x4D,0xB2,0xCD,0x32,0x2D,0xD2,0xAD,0x52,0x6D,0x92,0xED,0x12		;6x	
	db	0x1D,0xE2,0x9D,0x62,0x5D,0xA2,0xDD,0x22,0x3D,0xC2,0xBD,0x42,0x7D,0x82,0xFD,0x02		;7x
	db	0x03,0xFC,0x83,0x7C,0x43,0xBC,0xC3,0x3C,0x23,0xDC,0xA3,0x5C,0x63,0x9C,0xE3,0x1C		;8x
	db	0x13,0xEC,0x93,0x6C,0x53,0xAC,0xD3,0x2C,0x33,0xCC,0xB3,0x4C,0x73,0x8C,0xF3,0x0C		;9x	
	db	0x0B,0xF4,0x8B,0x74,0x4B,0xB4,0xCB,0x34,0x2B,0xD4,0xAB,0x54,0x6B,0x94,0xEB,0x14		;Ax
	db	0x1B,0xE4,0x9B,0x64,0x5B,0xA4,0xDB,0x24,0x3B,0xC4,0xBB,0x44,0x7B,0x84,0xFB,0x04		;Bx
	db	0x07,0xF8,0x87,0x78,0x47,0xB8,0xC7,0x38,0x27,0xD8,0xA7,0x58,0x67,0x98,0xE7,0x18		;Cx
	db	0x17,0xE8,0x97,0x68,0x57,0xA8,0xD7,0x28,0x37,0xC8,0xB7,0x48,0x77,0x88,0xF7,0x08		;Dx
	db	0x0F,0xF0,0x8F,0x70,0x4F,0xB0,0xCF,0x30,0x2F,0xD0,0xAF,0x50,0x6F,0x90,0xEF,0x10		;Ex
	db	0x1F,0xE0,0x9F,0x60,0x5F,0xA0,0xDF,0x20,0x3F,0xC0,0xBF,0x40,0x7F,0x80,0xFF,0x00		;Fx

LEDtab:		;lookup table used to form LED drive pattern
;		  0    1    2    3    4    5    6    7    8    9    A    B    C    D    E    F
	db	0xFF,0xFE,0xFE,0xFC,0xFC,0xFC,0xFC,0xFC,0xF8,0xF8,0xF8,0xF8,0xF8,0xF8,0xF8,0xF8		;0x
	db	0xF8,0xF8,0xF8,0xF8,0xF8,0xF8,0xF8,0xF8,0xF8,0xF8,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0		;1x
	db	0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xE0,0xE0		;2x
	db	0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0		;3x
	db	0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0		;4x
	db	0xE0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0		;5x	
	db	0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0		;6x	
	db	0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0,0xC0		;7x	
	db	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80		;8x
	db	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80		;9x
	db	0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80,0x80		;Ax
	db	0x80,0x80,0x80,0x80,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00		;Bx
	db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00		;Cx
	db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00		;Dx
	db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00		;Ex
	db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00		;Fx


copytab:			;copy reversing/absolute value pattern table into RAM
	movlw	LOW revtab
	movwf	TBLPTRL
	movlw	HIGH revtab
	movwf	TBLPTRH
	movlw	UPPER revtab
	movwf	TBLPTRU

	lfsr	1,RAMtab
cpylp:	TBLRD*			;read table byte	
	movf	TABLAT,W
	movwf	POSTINC1	;put it in RAM
	incfsz	TBLPTRL,F	;bump ptr
	bra	cpylp
	return
	
	

initser:			;sets up serial port
	movlw	0x10
	movwf	TXSTA		;slave, 8-bit, tx disabled, sync mode
	movlw	0x10
	movwf	BAUDCON		;SCKP=1, idle clk state is high
	clrf	RCSTA	
	bsf	RCSTA,SPEN	;enable ser port, 8-bit, sync, low speed
	bsf	RCSTA,RX9	;enable 9-bit reception
	movf	RCREG,W		;read data fifo
	movf	RCREG,W		;twice
	return


				;general-purpose delay routines
				;these use ttemp1, ttemp2, tbitcnt

wt100ms	movlw	D'250'		;kill 100 mSec
	goto	skip22

wt50ms	movlw	D'125'		;kill 50 mSec
	goto	skip22

wt20ms	movlw	D'50'		;kill 20 mSec
	goto	skip22

wt10ms	movlw	D'25'		;kill 10 mSec
	goto	skip22

wt2ms	movlw	D'5'		;kill 2 mSec
	goto	skip22

wt400uS	movlw	D'1'		;kill 400 uSec
skip22	movwf	ttemp2

wtolp	movlw	D'2'		;middle loop is 0.400 mSec
	movwf	tbitcnt
	
wtilp	movlw	D'222'		;1 cycle, delay for 200.4 uSec (40 MHz clock)
	movwf	ttemp1		;1 cycle	
dlyLp
	nop			;1 cycle
	nop			;1 cycle
	nop			;1 cycle
	nop			;1 cycle
	nop			;1 cycle
	nop			;1 cycle
	decfsz	ttemp1,F	;1 cycle, 2 if jump is taken
	goto	dlyLp		;2 cycles
				;inner loop total: ((222 * 9) + (1 * 4) + 2) * 0.100 = 200.4 uSec
	decfsz	tbitcnt,F	
	goto	wtilp		;inner loop until zero

	decfsz	ttemp2,F	;outer loop until zero
	goto	wtolp
	retlw	0x00

wt500ms	rcall	wt100ms
wt400ms	rcall	wt100ms
	rcall	wt100ms
wt200ms	rcall	wt100ms
	rcall	wt100ms
	return


;; Above this line are our subroutines
;; *****************************************************************************
			
				;This is the actual start of the program.
start:	
	clrwdt			

	clrf	INTCON2		;enable PORTB pullups, H->L edge for INT0

	movlw	0xC5
	movwf	T0CON		;timer 0 is 8-bit, 1:64 prescaler

	clrf	INTCON		;init intcon register, shut off all interrupts
	clrf	PIE1		;init PIE1 register, shut off all interrupts
	clrf	PIE2		;init PIE2 register, shut off all interrupts

	movlw	0x36		;turn off LED enables and overflow drives
	movwf	PORTA
	movlw	0x00
	movwf	PORTB		 
	movlw	0x3F		;turn off low 6 LED drives
	movwf	PORTC

	movlw	0x0F
	movwf	ADCON1		;set port A pins to digital mode

	movlw	PORTADIR	
	movwf	TRISA		;set up port A dir
	movlw	PORTBDIR	
	movwf	TRISB		;set up port B dir
	movlw	PORTCDIR
	movwf	TRISC		;set up port C dir

	rcall	wt10ms

	rcall	initser		;set up sync receive, not enabled yet

	rcall	copytab		;copy main table into RAM

mainlp:
	btfss	MTREN		;wait for meter enable to go high
	bra	mainlp

	bcf	INTCON,INT0IF	;clear flag
	bsf	INTCON,INT0IE	;enable MTREN H->L interrupt
	bcf	RCON,IPEN	;no int priority
	bsf	INTCON,GIE

	bsf	LEDENRb		;disable LED drives
	bsf	LEDENLb

	movlw	0x31
	movwf	T1CON		;timer1 resets rgt and lft peaks at 19 Hz rate

	lfsr	1,RAMtab	;point fsr1 to RAM table

	movlw	HIGH LEDtab	;and point table ptr to LED pattern table
	movwf	TBLPTRH
	movlw	UPPER LEDtab
	movwf	TBLPTRU

	bcf	RCSTA,CREN	;shut off receive
	movf	RCREG,W		;read data fifo
	movf	RCREG,W		;twice

	bcf	FLIPGATE	;don't invert LRCK to OR gate
				;get in sync with LRCK		
wthi1:  btfss	LRCK		
	bra	wthi1		;wait for real LRCK to be high

wtlow1:	btfsc	LRCK
	bra	wtlow1		;wait for H->L edge on real LRCK

wthi2:	btfss	LRCK		
	bra	wthi2		;wait for real LRCK to go high

	nop
	nop
	nop
	nop
	nop
	nop			;wait for delayed LRCK to be high (328 nSec at 48 KHz)
	bsf	RCSTA,CREN	;now enable continuous reception
				;but clock disabled until delayed LRCK goes low
				;cycle counts were 29, 29, 33, 31 with old code/hw setup

				;idea: Can we delay cont rec disable until after testing D9 bit?
				;this would eliminate shift into CY 
	
wtrgt0:	btfss	PIR1,RCIF	;3 wait for ser port rec'd character
	bra	wtrgt0		;1,2,4
	bsf	FLIPGATE	;5 disable clock
	rrcf	RCSTA,W		;6 save bit 0(D9), before we read data byte
	bcf	RCSTA,CREN	;7 disable continuous reception
	movf	RCREG,W		;8 grab data byte
	movwf	FSR1L		;9
	movf	RCREG,W		;10 grab second data byte, just in case
	bsf	RCSTA,CREN	;11 re-enable continuous reception
	movf	INDF1,W		;12 turn bits around, get absolute value
	btfsc	STATUS,C	;13
	xorlw	0x01		;14 toggle lsb based on D9
				;USART in PIC receives bit 0 first, bit 7 last, I2S is MSB first
				;so we need to turn byte around before processing

	cpfsgt  rgtpeak		;15 compare F with W, skip if F > W
	movwf	rgtpeak		;16 update peak if nec

	movf	rgtpeak,W 	;17			
	movwf	TBLPTRL		;18 point table ptr to LED pattern for right channel

	
wtlft0:	btfss	PIR1,RCIF	;3 wait for ser port rec'd character
	bra	wtlft0		;1,2,4
	bcf	FLIPGATE	;5 disable clock
	rrcf	RCSTA,W		;6 save bit 0
	bcf	RCSTA,CREN	;7 disable continuous reception
	movf	RCREG,W		;8 grab data byte
	movwf	FSR1L		;9
	movf	RCREG,W		;10 grab second data byte
	bsf	RCSTA,CREN	;11 safe to re-enable continuous receive at this pt
	movf	INDF1,W		;12 turn bits around, get absolute value
	btfsc	STATUS,C	;13
	xorlw	0x01		;14 toggle lsb based on D9

	cpfsgt  lftpeak		;15 compare F with W, skip if F > W
	movwf	lftpeak		;16 update peak if nec

		

wtrgt1:	btfss	PIR1,RCIF	;3 wait for ser port rec'd character
	bra	wtrgt1		;1,2,4
	bsf	FLIPGATE	;5 disable clock
	rrcf	RCSTA,W		;6 save bit 0
	bcf	RCSTA,CREN	;7 disable continuous reception
	movf	RCREG,W		;8 grab data byte
	movwf	FSR1L		;9
	movf	RCREG,W		;10 grab second data byte
	bsf	RCSTA,CREN	;11 safe to re-enable continuous receive here
	movf	INDF1,W		;12 turn bits around, get absolute value
	btfsc	STATUS,C	;13
	xorlw	0x01		;14 toggle lsb based on D9

	cpfsgt  rgtpeak		;15 compare F with W, skip if F > W
	movwf	rgtpeak		;16 update peak if nec


	TBLRD*			;17,18 get right LED pattern we set up before
	movf	TABLAT,W	;19
	movwf	PORTB		;20
	movwf	PORTC		;21 write out data to LED's		
	bcf	LEDENRb		;22 enable right LED bank
	

wtlft1:	btfss	PIR1,RCIF	;3 wait for ser port rec'd character
	bra	wtlft1		;1,2,4
	bcf	FLIPGATE	;5 disable clock
	rrcf	RCSTA,W		;6 save bit 0
	bcf	RCSTA,CREN	;7 disable continuous reception
	movf	RCREG,W		;8 grab data byte
	movwf	FSR1L		;9
	movf	RCREG,W		;10 grab second data byte
	bsf	RCSTA,CREN	;11 safe to re-enable continuous receive at this pt
	movf	INDF1,W		;12 turn bits around, get absolute value
	btfsc	STATUS,C	;13
	xorlw	0x01		;14 toggle lsb based on D9

	bsf	LEDENRb		;15 disable right LED bank

	cpfsgt  lftpeak		;16 compare F with W, skip if F > W
	movwf	lftpeak		;17

	movf	lftpeak,W	;18
	movwf	TBLPTRL 	;19 point table ptr to LED pattern for right channel
	TBLRD*			;20,21
	movf	TABLAT,W	;22
	movwf	PORTB		;23
	movwf	PORTC		;24 write out data to LED's		
	bcf	LEDENLb		;25 enable left LED bank

	
wtrgt2:	btfss	PIR1,RCIF	;3 wait for ser port rec'd character
	bra	wtrgt2		;1,2,4
	bsf	FLIPGATE	;5 disable clock
	movf	RCSTA,W		;6 save bit 0
	movwf	bit9temp	;7
	bcf	RCSTA,CREN	;8 disable continuous reception
	movf	RCREG,W		;9 grab data byte
	movwf	FSR1L		;10
	movf	RCREG,W		;11 grab second data byte
	bsf	RCSTA,CREN	;12 safe to re-enable continuous receive at this pt

	bsf	LEDENLb		;13 disable left LED bank

	btfss	PIR1,TMR1IF	;14 we'll get this at a 19 Hz rate
	bra	skpres1		;15 

				;flag is set, but don't clear it yet
	bsf	LEDRGTO		;16 test peak value, then reset it
	infsnz	rgtpeak,W	;17
	bcf	LEDRGTO		;18 turn on LED for next 50 mSec if rgtpeak = 0xFF
	clrf	rgtpeak		;19
	bra	wtlft2		;20,21 no need to check for new peak 
				;      if we're just going to clear it anyway
skpres1:	
	movf	INDF1,W		;17 turn bits around, get absolute value
	btfsc	bit9temp,0	;18
	xorlw	0x01		;19 flip LSB based on D9
	cpfsgt  rgtpeak		;20 compare F with W, skip if F > W
	movwf	rgtpeak		;21


wtlft2:	btfss	PIR1,RCIF	;3 wait for ser port rec'd character
	bra	wtlft2		;1,2,4
	bcf	FLIPGATE	;5 disable clock
	movf	RCSTA,W		;6 save bit 0
	movwf	bit9temp	;7
	bcf	RCSTA,CREN	;8 disable continuous reception
	movf	RCREG,W		;9 grab data byte
	movwf	FSR1L		;10
	movf	RCREG,W		;11 grab second data byte
	bsf	RCSTA,CREN	;12 safe to re-enable continuous receive at this pt

	btfss	PIR1,TMR1IF	;13 check timer flag again
	bra	skpres2		;14
	bcf	PIR1,TMR1IF	;15 now clear flag

	bsf	LEDLFTO		;16 test left peak value, then reset it
	infsnz	lftpeak,W	;17
	bcf	LEDLFTO		;18 turn on LED for next 50 mSec if lftpeak = 0xFF
	clrf	lftpeak		;19	
	bra	wtrgt0		;20,21	

skpres2:			;no need to check for new peak if we're just going to clear it anyway
	movf	INDF1,W		;16 turn bits around, get absolute value
	btfsc	bit9temp,0	;17
	xorlw	0x01		;18 flip LSB based on D9
	cpfsgt  lftpeak		;19 compare F with W, skip if F > W
	movwf	lftpeak		;20
	bra	wtrgt0		;21,22	
					
	END


		



	
	


